# mdi2.tcl---Mesytec MDI2 ADC module control frame



set frame "mdi2"
set frametitle "Mesytec MDI2 ADC Setup & Monitor"
set icontitle "mdi2"

set modulename mdi2
set moduletype  "MDI2"

loadsource common.vme


set mtm16 0
set bus 0

set maxadc 16
set adc  0

set alladcs 0
set allmtm16 0
set allbuses 0


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}
    
proc do_read_fifo {} {
  global CurrentVMEServer modulename module

  set info ""
  for {set Offset 0} {$Offset < 1088} {incr Offset 64} {

     set register $modulename#$module.DataMemory
     set data [string trimleft [read-vme-register $register]]

     append info "\nData Register (FIFO) Offset $Offset\n\n"
     foreach word $data {
         if {$word == 0xffffffff} then {
             append info "$word - assumed end of data\n"
             midas-information $info
             return
         }
         set type [expr $word >> 30 & 3]
         switch $type {
           0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3ff]
             set bus [expr $word >> 15 & 0x1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i,  bus= %1i;  adc= 0x%3x" $word $event_type $channel $bus $adc]"
           }
           1 {
             set event_type "header"
             set ID [expr $word >> 16 & 0xff]
             set WordCount [expr $word & 0x3ff]
             append info "[format "0x%8x   %8s    ModuleID= 0x%4x; WordCount= %3i" $word $event_type $ID $WordCount]"
           }
           2 {
             set event_type "bad"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
           3 {
             set event_type "end event"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
         }
         append info "\n"
     }
  }
  append info "end of fifo reached\n"
  midas-information $info
}

proc show_registers {} {
   global EGreport EGallow modulename module registers_table

   MDI2_registers               ;#  now registers_table in global variable

   clear-footer
   set EGreport -1

   set info ""

   set EGallow 0x2000a

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $modulename#$module.$register]
       if {$data == $EGallow} then {set data "Bus Error"}
       append info "[format "%-40s %-20s" $description $data]\n"
   }

   set EGallow 0xffffffff

   global _information_font
   set _information_font fixed
   midas-information $info

   set EGreport 0
}

proc show_software {} { inform mdi2sftwre start}

proc show_electronics {} {

 set valuex 32c

  :frame [self frame-name].electronics :label "MDI2 Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5; :col-gap 10

      :text DATALEN     :label "Buffer data length" :w 8 :valuex $valuex  :deaf T
      :checkbox S0      :label "Data Available" :valuex $valuex :deaf T
      :choice S1        :label "Acquisition State" :strings [list Stop Start] :w 5 :valuex $valuex :deaf T
      :choice SEQENABLE    :label  "Sequencer Enable" :strings [list 0 1] :w 3 :valuex $valuex :exclusive F
      :choice TRIG_SRC0 :label "Trigger Source Bus 0" :strings [list common trigger1 trigger0]  :valuex $valuex :exclusive F
      :choice TRIG_SRC1 :label "Trigger Source Bus 1" :strings [list common trigger1 trigger0]  :valuex $valuex :exclusive F
      :choice TRIG_COMM :label "Common Trigger Source" :strings [list trigger0 trigger1]  :valuex $valuex :exclusive F
      :checkbox S2      :label "Bus watchdog enable"  :valuex $valuex :deaf T
      :choice CLKFREQ0  :label "Clock Frequency Bus 0 (MHz)" :strings [list 1.25 2.5 5 10] :w 3  :valuex $valuex
      :choice CLKFREQ1  :label "Clock Frequency Bus 1 (MHz)" :strings [list 1.25 2.5 5 10] :w 3  :valuex $valuex
      :checkbox S3      :label "Sequencer Busy" :valuex $valuex :deaf T

     :layout H; :next-row T

     :button EREDISPLAY :label "Update"            :command electronics_redisplay
     :button R1         :label "Module Soft Reset" :command "do_regcmd SoftReset"
     :button R2         :label "Readout Reset"     :command "do_regcmd RdOutReset"
     :button R3         :label "Fifo Initialise"   :command "do_regcmd FifoInit"
      :next-row T
     :button R4         :label "Generate Software Trigger"   :command "do_regcmd GenTrig"
     :button R5         :label "Generate Test Event"   :command "do_regcmd GenEvent"
     :button R6         :label "Frontend Reset"   :command "do_regcmd FrontendReset"

  :show T

  electronics_redisplay
}

proc do_regcmd {r} {
   global modulename module

   switch $r {
       SoftReset    {write-vme-register $modulename#$module.Reset 1}
       RdOutReset   {write-vme-register $modulename#$module.RdOut_Reset 1}
       FifoInit     {write-vme-register $modulename#$module.Fifo_Reset 1}
       GenTrig      {write-vme-register $modulename#$module.Gen_Trig 1}
       GenEvent     {write-vme-register $modulename#$module.Gen_Event 1}
       FrontendReset     {write-vme-register $modulename#$module.frontend_reset 1}

   }
}

proc electronics_redisplay {} {
   global modulename module

    :modify DATALEN :value [read-vme-register $modulename#$module.Data_Len]
    :modify S0      :value [read-vme-register $modulename#$module.Data_Rdy]
    :modify S1      :value [read-vme-register $modulename#$module.Acq_Ctrl]
    :modify S2      :value [read-vme-register $modulename#$module.bus_watchdog]
    :modify S3      :value [read-vme-register $modulename#$module.seq_busy]

    :modify SEQENABLE   :value [read-vme-register $modulename#$module.seq_enable]
    :modify TRIG_SRC0   :value [read-vme-register $modulename#$module.trig_source0]
    :modify TRIG_SRC1   :value [read-vme-register $modulename#$module.trig_source1]
    :modify TRIG_COMM   :value [read-vme-register $modulename#$module.com_trig_source]
    :modify CLKFREQ0    :value [read-vme-register $modulename#$module.seq_clk_freq0]
    :modify CLKFREQ1    :value [read-vme-register $modulename#$module.seq_clk_freq1]

}

proc click-threshold {adc vl} {
   global modulename module
   global mtm16 bus
   global allmtm16 allbuses alladcs

   for {set m 0} {$m <= 7} {incr m} {
       if {$allmtm16 == 0 && $m != $mtm16} {continue}

      for {set b 0} {$b <= 1} {incr b} { 
          if {$allbuses == 0 && $b != $bus} {continue}

         for {set w 0} {$w < 16} {incr w} {
             if {$alladcs == 0 && $w != $adc} {continue}

                set i [expr ($m * 32) + ($w * 2) + $b]
                write-vme-register $modulename#$module.Threshold$i $vl
                set-footer "threshold $i set to $vl"
         }
      }
   }
}

proc redisplay_thresholds {} {
   global EGreport
    global modulename module
    global mtm16 bus

   clear-footer
   set KEEP $EGreport
   set EGreport -1
   set-busy

#    redisplay the current thresholds

   for {set w 0} {$w < 16} {incr w} {
       set i [expr ($mtm16 * 32) + ($w * 2) + $bus]
       set vl [read-vme-register $modulename#$module.Threshold$i]
       :modify THRESHOLD$w  :value $vl
   }

   clear-busy
   set EGreport $KEEP
}

proc click-redisplay {} {
    global modulename module

    :modify GATEDELAY0  :value [read-vme-register $modulename#$module.hold_delay0]
    :modify GATEDELAY1  :value [read-vme-register $modulename#$module.hold_delay1]
    :modify GATEWIDTH0  :value [expr [read-vme-register $modulename#$module.hold_width0]*25]
    :modify GATEWIDTH1  :value [expr [read-vme-register $modulename#$module.hold_width1]*25]

    redisplay_thresholds

}

set mtm16_gain 0
set mtm16_pol 0
set mtm16_trig 0
set mtm16_thres 0

proc do_mtm16 {} {
  global mtm16_gain mtm16_pol mtm16_trig mtm16_thres

 set valuex 16c

  :frame [self frame-name].mtm16 :label "Mesytec MTM16 Commands" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5; :col-gap 10

      :text RCBUSNO     :label "RC Busno"  :w 3 :valuex $valuex :deaf T
      :text RCMODNUM    :label "RC Modnum" :w 3 :valuex $valuex :deaf T
      :text RCOPCODE    :label "RC OpCode" :w 3 :valuex $valuex :deaf T
      :text RCADDR      :label "RC Addr"   :w 3 :valuex $valuex :deaf T
      :text RCDAT       :label "RC Data"   :w 6 :valuex $valuex :deaf T
      :text RCSTAT      :label "RC Status" :w 3 :valuex $valuex :deaf T

      :next-row T

      :choice MTM16GAIN        :label "Gain" :strings [list High Low] :w 5 :valuex $valuex :value $mtm16_gain
      :choice MTM16POL         :label "Input Polarity" :strings [list Neg Pos] :w 5 :valuex $valuex :value $mtm16_pol
      :checkbox MTM16TRIG      :label "Trigger Enable"  :valuex $valuex :value $mtm16_trig
      :slider MTM16THRES       :label "Common Threshold" :min 0 :max 8191 :w 5 :valuex $valuex :value $mtm16_thres

      :next-row T

     :button MTM16REDISPLAY    :label "Update"  :command mtm16_redisplay

   :show T

   mtm16_redisplay
}

proc mtm16_redisplay {} {
   global modulename module

    :modify RCBUSNO  :value [read-vme-register $modulename#$module.rc_busno]
    :modify RCMODNUM :value [read-vme-register $modulename#$module.rc_modnum]
    :modify RCOPCODE :value [read-vme-register $modulename#$module.rc_opcode]
    :modify RCADDR   :value [read-vme-register $modulename#$module.rc_adr]
    :modify RCDAT    :value [read-vme-register $modulename#$module.rc_dat]
    :modify RCSTAT   :value [read-vme-register $modulename#$module.rc_status]
}

proc write_mtm16thres {vl} {
   global modulename module
   global mtm16 bus
   global allmtm16 allbuses alladcs

   write-vme-register $modulename#$module.rc_opcode 3    ;#   RC on

   for {set m 0} {$m <= 7} {incr m} {
       if {$allmtm16 == 0 && $m != $mtm16} {continue}

      for {set b 0} {$b <= 1} {incr b} { 
          if {$allbuses == 0 && $b != $bus} {continue}

          write-vme-register $modulename#$module.rc_busno $b
          write-vme-register $modulename#$module.rc_modnum $m

          write-vme-register $modulename#$module.rc_opcode 16   ;#   write data

          write-vme-register $modulename#$module.rc_adr 1       ;#   set common threshold
          write-vme-register $modulename#$module.rc_dat $vl
          after 1
      }
   }

   write-vme-register $modulename#$module.rc_opcode 4    ;#   RC off
}

proc write_mtm16 {} {
   global modulename module
   global mtm16 bus
   global allmtm16 allbuses alladcs
  global mtm16_gain mtm16_pol mtm16_trig

   write-vme-register $modulename#$module.rc_opcode 3    ;#   RC on

   for {set m 0} {$m <= 7} {incr m} {
       if {$allmtm16 == 0 && $m != $mtm16} {continue}

      for {set b 0} {$b <= 1} {incr b} { 
          if {$allbuses == 0 && $b != $bus} {continue}

          write-vme-register $modulename#$module.rc_busno $b
          write-vme-register $modulename#$module.rc_modnum $m

          write-vme-register $modulename#$module.rc_opcode 16   ;#   write data

          write-vme-register $modulename#$module.rc_adr 0
          write-vme-register $modulename#$module.rc_dat [expr $mtm16_gain + ($mtm16_pol << 1) + ($mtm16_trig << 2)]
          after 1
      }
   }

   write-vme-register $modulename#$module.rc_opcode 4    ;#   RC off
}

proc click arg {
    global debug actionlist
    global modulename module
    global mtm16 bus
    global allmtm16 allbuses alladcs
    global mtm16_gain mtm16_pol mtm16_trig mtm16_thres
#insert-debug "click with argument \{$arg\}"
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MTM16-SEL      -
      MTM16-NUM   {set mtm16 $v; :modify MTM16-NUM :value $v; click-redisplay}

      BUS-SEL      -
      BUS-NUM     {set bus $v; :modify BUS-NUM :value $v; click-redisplay}

      ALLMTM16    {set allmtm16 $v}
      ALLBUS      {set allbuses $v}
      ALL-ADCS    {set alladcs $v}

      THRESHOLD*  {click-threshold [string range $w 9 end] $v}
      SEQENABLE   {write-vme-register $modulename#$module.seq_enable $v}
      TRIG_SRC0   {write-vme-register $modulename#$module.trig_source0 $v}
      TRIG_SRC1   {write-vme-register $modulename#$module.trig_source1 $v}
      TRIG_COMM   {write-vme-register $modulename#$module.com_trig_source $v}
      GATEDELAY0  {write-vme-register $modulename#$module.hold_delay0 $v}
      GATEDELAY1  {write-vme-register $modulename#$module.hold_delay1 $v}
      GATEWIDTH0  {write-vme-register $modulename#$module.hold_width0 [expr $v/25]}
      GATEWIDTH1  {write-vme-register $modulename#$module.hold_width1 [expr $v/25]}
      CLKFREQ0    {write-vme-register $modulename#$module.seq_clk_freq0 $v}
      CLKFREQ1    {write-vme-register $modulename#$module.seq_clk_freq1 $v}

      MTM16THRES  {set mtm16_thres $v; write_mtm16thres $v}
      MTM16GAIN   {set mtm16_gain $v; write_mtm16}
      MTM16POL    {set mtm16_pol $v; write_mtm16}
      MTM16TRIG   {set mtm16_trig $v; write_mtm16}

      REDISPLAY  	{click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env actionlist
  global mtm16 bus maxadc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MTM16-NUM  :label "MTM16 Module number:" :min 0 :max 15 :w 3 :value $mtm16
     :menubutton MTM16-SEL :label "select:" :strings [list 0 1 2 3 4 5 6 7]

     :number BUS-NUM  :label "Bus number:" :min 0 :max 1 :w 2 :value $bus
     :menubutton BUS-SEL :label "select:" :strings [list 0 1]

     :hskip 50
     :checkbox ALLMTM16 :label "Act on ALL modules?"
     :checkbox ALLBUS   :label "Act on BOTH buses?"

     :next-row T
     :message M1 :label "Module Settings" :bold T
     :next-row T
     :slider GATEDELAY0 :label "Gate Delay Bus 0 (ns)" :min 0 :max 8191
     :slider GATEDELAY1 :label "Gate Delay Bus 1 (ns)" :min 0 :max 8191
     :next-row T
     :slider GATEWIDTH0 :label "Gate Width Bus 0 (ns)" :min 0 :max 3175 :resolution 25
     :slider GATEWIDTH1 :label "Gate Width Bus 1 (ns)" :min 0 :max 3175 :resolution 25


  
   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 0
     :message M2 :label "ADC Thresholds" :bold T
     :hskip 400
     :checkbox ALL-ADCS :label "Act on ALL ADCs?"
     :next-row T

     for {set i 0} {$i < $maxadc} {} {
         :slider   THRESHOLD$i :label "$i" :min 0 :max 4095 :entry-width 4 :valuex 2c
         :hskip 10
         incr i
         if {[expr $i / 4 * 4] == $i} {:next-row T}
     }
  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button REGISTERS   :label "Show registers"  :command show_registers
      :button ELECTRONICS :label "Electronics"     :command show_electronics
      :button SOFTWARE    :label "Software"        :command show_software
      :button MEMRD       :label "read data FIFO"  :command do_read_fifo
      :button MTM16       :label "MTM16 Commands"  :command do_mtm16


  :show T

}

