

proc Set_MDI2_Options {} {
    global Options MDI2_Base

    load_Options

   if {![info exists Options(mdi2.channels)]} {set Options(mdi2.channels) 4096}
   if {![info exists Options(mdi2.offset)]} {set Options(mdi2.offset) 0}
   if {![info exists Options(mdi2.shift)]} {set Options(mdi2.shift) 0}
   if {![info exists Options(mdi2_GroupBase)]} {set Options(mdi2_GroupBase) $MDI2_Base}
   set MDI2_Base  $Options(mdi2_GroupBase)

    update_Options
}

proc Dump_Raw_VME_Data_MDI2 {} {

   set info ""

   set maxmodule [inform vmecontrol inquire mdi2count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nMDI2#$i\n"
      execute-command 19 "Getting Raw Data for MDI2#$i" "10 $i" 2
      set data [read-register NetVar.EXEC.Response]

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }

         set type [expr $word >> 30 & 3]
         switch $type {
           0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3ff]
             set bus [expr $word >> 15 & 0x1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i,  bus= %1i;  adc= 0x%3x" $word $event_type $channel $bus $adc]"
           }
           1 {
             set event_type "header"
             set ID [expr $word >> 16 & 0xff]
             set WordCount [expr $word & 0x3ff]
             append info "[format "0x%8x   %8s    ModuleID= 0x%4x; WordCount= %3i" $word $event_type $ID $WordCount]"
           }
           2 {
             set event_type "bad"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
           3 {
             set event_type "end event"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
         }
         append info "\n"
     }
   }
   return $info
}

proc define_MDI2_config {} {
    global VMEserver
    global MDI2_Base

   set maxmodule [inform vmecontrol inquire mdi2count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "2 0 [format "0x%x" $MDI2_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/mdi2#$i.Reset'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register mdi2#$i.Reset"
          clear-footer
          error $m
      }
      DefineConfig "2 $i [string range [first $m] 2 end]"
   }
}

proc create_MDI2_spectra {} {
    global SpectrumServer MDI2_Base
    global Options

   set maxmodule [inform vmecontrol inquire mdi2count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 16

   set maxbus 2
   set maxmtm16 8

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module mdi2#$i"

   for {set k 0} {$k < $maxbus} {incr k} {

   for {set l 0} {$l < $maxmtm16} {incr l} {

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName mdi2#$i.b$k.m$l.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(mdi2.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          set group [expr $MDI2_Base + (($i - 1) * 32) + ($k * 16) + $l]
          map_spectrum  "$spectrumname $group $j $Options(mdi2.offset) $Options(mdi2.shift)"
      }
    }
   }
   }
   }
}

